function [HG] = Hmaker(node, elem, Elength, Etheta, m, M, nelems, nnodes, HG, Glength)
% Function Hmaker to make gloabl H matrix by calculating the
% local matrix and then adding to the global matrix.
% Thomas Reeve and Boyang Tang
% ME 599-3, Winter 2006
% Last Modified 4/18/06

%Used to add 1/Mx and 1/My to the elemental matrix. Used to ensure the H
%matrix is not singular
add=[[1 0 0 0 0 0 0];
     [0 1 0 0 0 0 0];
     [0 0 0 0 0 0 0];
     [0 0 0 1 0 0 0];
     [0 0 0 0 1 0 0];
     [0 0 0 0 0 0 0];
     [0 0 0 0 0 0 0]];
 
% Start Loop to Update Global H matrix
for qg=1:nelems
        
    % Figure out which element and nodes operating on
    % Find the node numbers to figure out where in global matrix to add them
    node1 = elem(qg,1);
    node2 = elem(qg,2);
    
    % Declare the constants for this element
    l=Elength(qg);
    S = sin(Etheta(qg));
    C = cos(Etheta(qg));
    u = (m*((Elength(qg)^2)))/(5*M);
    
    %Clear the elemental H matrix
    He = zeros(7);
    
    % Determine Elemental Viscosity Matrix
    He=[[2*l^2*S^2 -2*l^2*S*C  5*l*S  l^2*S^2 -l^2*S*C -l*S -4*l*S];
            [-2*l^2*S*C 2*l^2*C^2 -5*l*C -l^2*S*C  l^2*C^2  l*C 4*l*C];
            [5*l*S -5*l*S 14+4*u l*S -l*C 2-u -16+2*u];
            [l^2*S^2 -l^2*S*C l*S 2*l^2*S^2 -2*l^2*S*C -5*l*S 4*l*S];
            [-l^2*S*C l^2*C^2 -l*C -2*l^2*S*C 2*l^2*C^2 5*l*C -4*l*C];
            [-l*S l*C 2-u -5*l*C 5*l*C 14+4*u -16+2*u];
            [-4*l*S 4*l*C -16+2*u 4*l*S -4*l*C -16+2*u 32+16*u]];
        
    He=(1/(6*m*l))*He;
    
    %Add the small amount to discourage singularity
    He=He+(10e-6)*add;
    
    %Now assemble the elemental matrix into the global one
     if qg==1 %First element
        HG(1:7,1:7)=HG(1:7,1:7)+He(:,:);
     elseif qg==2 %Second element
        HG(4:6,4:6)=HG(4:6,4:6)+He(1:3,1:3);
        HG(4:6,8:11)=HG(4:6,8:11)+He(1:3,4:7);
        HG(8:11,4:6)=HG(8:11,4:6)+He(4:7,1:3);
        HG(8:11,8:11)=HG(8:11,8:11)+He(4:7,4:7);
     elseif node2==nnodes %Last element
        HG(1:3,1:3)=HG(1:3,1:3)+He(4:6,4:6);
        HG(Glength-4:Glength-2,Glength-4:Glength-2)=HG(Glength-4:Glength-2,Glength-4:Glength-2)+He(1:3,1:3);
        HG(Glength,Glength)=HG(Glength,Glength)+He(7,7);
        HG(1:3,Glength-4:Glength-2)=HG(1:3,Glength-4:Glength-2)+He(4:6,1:3);
        HG(1:3,Glength)=HG(1:3,Glength)+He(4:6,7);
        HG(Glength-4:Glength-2,1:3)=HG(Glength-4:Glength-2,1:3)+He(1:3,4:6);
        HG(Glength,1:3)=HG(Glength,1:3)+He(7,4:6);
        HG(Glength-4:Glength-2,Glength)=HG(Glength-4:Glength-2,Glength)+He(1:3,7);
        HG(Glength,Glength-4:Glength-2)=HG(Glength,Glength-4:Glength-2)+He(7,1:3);
     else %Other elements
        HG(4*(qg-1):4*(qg-1)+2,4*(qg-1):4*(qg-1)+2)=HG(4*(qg-1):4*(qg-1)+2,4*(qg-1):4*(qg-1)+2)+He(1:3,1:3);
        HG(4*(qg-1):4*(qg-1)+2,4*qg:4*qg+3)=HG(4*(qg-1):4*(qg-1)+2,4*qg:4*qg+3)+He(1:3,4:7);
        HG(4*qg:4*qg+3,4*(qg-1):4*(qg-1)+2)=HG(4*qg:4*qg+3,4*(qg-1):4*(qg-1)+2)+He(4:7,1:3);     
        HG(4*qg:4*qg+3,4*qg:4*qg+3)=HG(4*qg:4*qg+3,4*qg:4*qg+3)+He(4:7,4:7);
     end
     
end
