function [node, elem, nnodes, nelems, right] = bubblecheck(node, elem, nelems, lmerge, nbuffer, nnodes, right)
% Function bubblecheck to see if nodes get close enough to create a droplet
% and if so, to rearrange nodes and elements so that bubble part is on the
% bottom end of the node/element list
% lmerge: merging distance, nbuffer: min number of nodes needed to be
% between merging nodes to constitute a droplet
% Thomas Reeve and Boyang Tang
% ME 599-3, Winter 2006
% Last Modified 4/21/06

% Reset Bubble checking variables
nmerge = 0;
elstart = 0;
elend = 0;
n1 = 0;
n2 = 0;
n3 = 0;
n4=0;
d =0;
droplength = 0;

% Goes through every node except starting one and ones less than buffer
% distance from the other end
for h=2:nelems
    node1 = elem(h,1);
    node2 = elem(h,2);
    
    % Start Checking at end node of the element
    x2 = node(node2,1);
    y2 = node(node2,2);
    
    % Go through the other elements
    for q=h+nbuffer:nelems-2
        node3 = elem(q,1);
        node4 = elem(q,2);
        
        % Check at the beginning of the element for a merge
        x3 = node(node3,1);
        y3 = node(node3,2);
        
        % Check distance
        dist = sqrt( ((x3-x2)^2) + ((y3-y2)^2) );
        %dist = abs(x3-x2);
        
        % Mark the elements and nodes if within merging distance
        if (dist < lmerge)
            nmerge=nmerge+1;
            elstart(nmerge) = h;
            elend(nmerge) = q;
            n1(nmerge) = node1;
            n2(nmerge) = node2;
            n3(nmerge) = node3;
            n4(nmerge) = node4;
            d(nmerge) = dist;
            droplength(nmerge) = node3 - node1;
        end
    end
end

nelems=nelems;
nnodes=nnodes;
right=right;
elem=elem;
node=node;

if (nmerge > 0)
    % Find minimum merge distance and go with that one
    [y, g] = min(d);
    dl = droplength(g);
    nw1 = n1(g);
    nb1 = n2(g);
    nb2 = n3(g);
    nw2 = n4(g);
    
    % Remove the Droplet Nodes from the node listings and move them to a
    % temp slot
    for t=1:dl 
        delnode = nw2 - t;
        temp(t,1) = node(delnode,1);
        temp(t,2) = node(delnode,2);
        node(delnode, :) = [];
    end
    
    % Recheck Number of Nodes and Make the Last one the right end point
    % node
    nnodes = size(node,1);
    right = nnodes; % Mark which is the right endpoint
    
    % Reconnect Wall Elements together
    for w=1:nnodes-1
        elem(w,1) = w;
        elem(w,2) = w+1;
    end
    
    % Add droplet nodes back to the node listing at the end
    for v=(nnodes+1):(nnodes+dl)
        node(v,1) = temp(v-nnodes,1);
        node(v,2) = temp(v-nnodes,2);
    end
    
    % Connect Droplet Elements Together
    for q=nnodes:(nnodes+dl-2)
        elem(q,1) = q+1;
        elem(q,2) = q+2;
    end
    
    nnodes = size(node,1);
    
    % Connect Last drop point to first drop point
    elem(nnodes-1,1) = (nnodes-1);
    elem(nnodes-1,2) = (right+1);
    
    % Recheck Total length of nodes and elements
    nnodes = size(node,1);
    nelems = size(elem,1);
end
        