% Script Command mFile for ME 599-3 Nanoscale Diffusion Solid Droplet Project
% To run this project, run this file by typing 'drop' and hitting enter
% Thomas Reeve and Boyang Tang
% ME 599-3, Winter 2006
% Last modified 4/21/06

% Cleaning Stuff
clear all
close all
clc

% Load the constants
load constants.mat

% Load the node.mat file for nodal positions
load node.mat
% Find the number of nodes
nnodes = size(node,1);
right = nnodes;

% Create Elements Assuming Initially All Linked Together in Order
for ej=1:(nnodes-1)
    elem(ej,1)=ej;
    elem(ej,2)=ej+1;
end
% Find the number of elements
nelems =  size(elem,1);

% Set intial time
t = 0.0;

% ~~~~~~~~~ Movie Stuff ~~~~~~~~~~~~~~~~~
%Set Movie Frame Counter
counter = 1;

%Initialize the vectors used to plot the shape
x={};
y={};

%Set the counter for the movie frames
counter=1;

%Save the initial nodal positions
x{counter}=node(1:nnodes,1);
y{counter}=node(1:nnodes,2);

%Find the inital min and max for printing
xmin=min(x{counter});
xmax=max(x{counter});
ymin=min(y{counter});
ymax=max(y{counter});

% Save time for printing later
savet(counter)=0;


% ~~~~~~~~~~~~ END MOVIE STUFF ~~~~~~~~~~~~~~~~~~~~~~~~~

while (t<tmax)

    % Find Length and Angles of the Elements
    [Elength, Etheta] = lengle(node, elem, nelems);
    
    % Enable if part if getting weird droplets off droplets effect, will force it to
    % only to check for a merge once to keep stable
    %if right==nnodes
        % Collision Check for droplet formation
        [node, elem, nnodes, nelems, right] = bubblecheck(node, elem, nelems, lmerge, nbuffer, nnodes, right);
    %end
    
    % Check for elements that are too short or too long and fix them if
    % need be
    if ((min(Elength)<lmin) | (max(Elength)>lmax))
        while (min(Elength)<lmin)
            % Fix the Lengths
            [node, elem, nnodes, nelems, right] = lengthcheck(node, elem, lmin, lmax, nelems, nnodes, Elength, Etheta, right);
            
            % Refind the new Lengths and Angles
            [Elength, Etheta] = lengle(node, elem, nelems);
        end
    end
    
    % To ensure that two elements are not inline with each other, assign a
    % random deviation to the x and y coordinates of a random number between 
    % 0.99 and 1.01 times the current position
    if abs(Etheta(1)-Etheta(nelems))<0.05
        node(1,:)=node(1,:)*(0.99+(1.01-0.99)*rand);
    end
    for i=2:nelems-1
        if abs(Etheta(i)-Etheta(i-1))<0.05
            node(i,:)=node(i,:)*(0.99+(1.01-0.99)*rand);    
        end
    end
    
    % Find Length and Angles of the Elements
    [Elength, Etheta] = lengle(node, elem, nelems);
    % Find the number of nodes
    nnodes = size(node,1);
    % Find the number of elements
    nelems = size(elem,1);
    
    % Find Length of Global Matrix Sides (x,y,j per node, and jm per elem)
    GLength = (3*nnodes)+nelems;
    
    %Sometimes this error will print. Use this command to turn it off
    warning off MATLAB:nearlySingularMatrix;
    
    % Declare Empty Global H and F Matrices
    HG = zeros(GLength);
    FG = zeros(GLength,1);
    
    % Calculate and Assign the Global H and F Matrices
    HG = Hmaker(node, elem, Elength, Etheta, littlem, bigM, nelems, nnodes, HG, GLength);
    FG = Fmaker(node, elem, Etheta, gamma, nelems, nnodes, FG, GLength);
    
    % Crop Endpoint Rows and Columns for Boundary Conditions
    % Remove x'n, y'n from HG
      HG(:,(4*(right-1)+1)) = [];
      HG(:,(4*(right-1))) = [];
      HG((4*(right-1)+1),:) = [];
      HG((4*(right-1)),:) = [];

    %Remove x1', y1' from HG
    HG(:,2) = [];
    HG(:,1) = [];
    HG(2,:) = [];
    HG(1,:) = [];
    
    % Remove f1 fn
    FG(4*(right-1)+1) = [];
    FG(4*(right-1)) = [];
    FG(2) = [];
    FG(1) = [];
    
    % Calculate Global Q Vector
    qG = HG\FG;
    
    %Extract the nodal velocities
    qdot = zeros(nnodes,2);
    
    for j=1:nnodes
        if j==1          % Fixes Starting Node, boundary condition
            qdot(j,1)= 0;
            qdot(j,2)= 0;
        elseif j==right % Fixes Last Node, boundary condition
            qdot(j,1)= 0;
            qdot(j,2)= 0;
        elseif (abs(qG((4*(j-1))-2)) > 1)
            qdot(j,1) = 0;
        elseif (abs(qG((4*(j-1))+1-2)) > 1)
            qdot(j,2) = 0;
        elseif j>right
            qdot(j,1)=qG((4*(j-1))-4);
            qdot(j,2)=qG((4*(j-1))-3);
        else
            qdot(j,1)=qG((4*(j-1))-2);
            qdot(j,2)=qG((4*(j-1))-1);
        end
        
    end
    % Update the nodal positons
    node=node+qdot*deltat;
    
    % Update time and movie counter
    t = t + deltat;
    counter = counter+1;
    disp('On time frame');
    disp(t);
    
    % ~~~~~~~~~~~~~ MOVIE STUFF ~~~~~~~~~~~~~~~~~~~
    %Save the nodal positions for printing later
        x{counter}=node(1:nnodes,1);
        y{counter}=node(1:nnodes,2);
               
        %Find the maximum to set the printout later
        if max(x{counter})>xmax
            xmax=max(x{counter});
        end
        if min(x{counter})<xmin
            xmin=min(x{counter});
        end
        if max(y{counter})>ymax
            ymax=max(y{counter});
        end
        if min(y{counter})<ymin
            ymin=min(y{counter});
        end
        
        savet(counter)=t;
   % ~~~~~~~~~~~~ END MOVIE STUFF ~~~~~~~~~~~~~~~~~
    
end % End of main while loop


%~~~~~~~~~~~~~~ MOVIE STUFF ~~~~~~~~~~~~~~~~~~~

%Pad the sides of the plot
xsize=xmax-xmin;
ysize=ymax-ymin;
xmin=xmin-xsize/10;
xmax=xmax+xsize/10;
ymin=ymin-ysize/10;
ymax=ymax+ysize/10;


%Save the file in case something goes wrong with the AVI creation, the data
%is not lost.

disp('Saving the data')
%save savefile xmin xmax ymin ymax x y savet;

%Plot out the nodal positions from the simulation
close all;
figure;
format short;
plot(x{1},y{1},'ko')
axis([xmin xmax ymin ymax]);
printtitle=['Time is: ',num2str(savet(1),'%0.7g'),' sec'];
text(xmin+xsize/20,ymax-ysize/20,printtitle);
frames(1)=getframe;
for i=2:size(x,2)
    plot(x{i},y{i},'ko');
    axis([xmin xmax ymin ymax]);
    printtitle=['Time is: ',num2str(savet(i),'%0.7g'),' sec'];
    text(xmin+xsize/20,ymax-ysize/20,printtitle);
    frames(i)=getframe;
end

%Write the AVI file
movie2avi(frames,'moviefile', 'quality', 30);