function [node, elem, nnodes, nelems, right] = lengthcheck(node, elem, lmin, lmax, nelems, nnodes, Elength, Etheta, right)
% Function lengthcheck to check if the element lengths are within specified
% maxes and mins, and fix the elements if they are too short or too long
% Thomas Reeve and Boyang Tang
% ME 599-3, Winter 2006
% Last Modified 4/21/06

%If any elements are too small, then fix them by deleting the middle two
%nodes
if (min(Elength)<lmin) % Check to see if any elements are less than the minimum length allowed
    
    % Fix the smallest element
    [y,i] = min(Elength);
    delnode = elem(i,2);
    
    
    if (delnode > right) % Case if it was a droplet node that was deleted
        node(delnode,:) = [];
        elem(delnode,:) = [];
        nnodes = size(node,1);
        
        for v = right:(nnodes-2)
            elem(v,1) = v+1;
            elem(v,2) = v+2;
        end
        elem(nnodes-1,1) = nnodes;
        elem(nnodes-1,2) = right+1;
    % Case if it was a wall node that was deleted after a droplet formed
    elseif (delnode < right) && (right < nnodes)
        node(delnode, :) = [];
        elem(delnode,:) = [];
        nnodes = size(node,1);
        right = right-1; % Update right endpoint
        
        % Connect Wall Elements
        for v=1:right-1
            elem(v,1) = v;
            elem(v,2) = v+1;
        end
        
        % Connect Droplet Elements
        for k=right:(nnodes-2)
            elem(k,1)=k+1;
            elem(k,2)=k+2;
        end
        elem(nnodes-1,1) = nnodes;
        elem(nnodes-1,2) = right+1;
        nelems=size(elem,1);
        
    else % Standard Case, no droplet yet
        [y,i]=min(Elength);
        delnode=elem(i,2);
        node(delnode,:)=[];
        elem(nelems,:)=[];
        
        nelems = size(elem,1);
        nnodes = size(node,1);
        right = nnodes; % Update right endpoint

        elem(nelems,2)= nnodes;  
    end
end

% %If any elements are too big, then split them in half by making a new node
% %in the middle of the big one
if (max(Elength)>lmax)
    [y,i]= max(Elength);
    
    node1=elem(i-1,1);
    node2=elem(i-1,2);
    
    x1=node(node1,1);
    y1=node(node1,2);
    x2=node(node2,1);
    y2=node(node2,2);
    
    % Case if a bubble element that was too long
    if (i > (right+1))
        temp=zeros(nnodes+1,2);
        
        % Insert a new element into the node listings
        temp(1:node1,:)=node(1:node1,:);    
        temp(node1+1,1)=(x1+x2)/2;
        temp(node1+1,2)=(y1+y2)/2;    
        temp(node2+1:nnodes+1,:)=node(node2:nnodes,:);
        node=temp;
        
        % Reconnect Elements of the wall
        for v=1:right-1
            elem(v,1) = v;
            elem(v,2) = v+1;
        end
        
        % Connect Droplet Elements
        for k=right:(nnodes-2)
            elem(k,1)=k+1;
            elem(k,2)=k+2;
        end
        elem(nnodes-1,1) = nnodes;
        elem(nnodes-1,2) = right+1;
        
        nelems = size(elem,1);
        nnodes = size(node,1);
        
    % Case where a wall segment is too long after a bubble has formed 
    elseif (i<right) && (right < nnodes)
        temp=zeros(nnodes+1,2);
        newnode(1)=(x1+x2)/2;
        newnode(2)=(y1+y2)/2;
        
        for tl=1:i % Adds prelong one nodes to a temp list
            temp(tl,1)=node(tl,1);
            temp(tl,2)=node(tl,2);
        end
        % Adds new middle node to temp list
        temp(i+1,1)=newnode(1);
        temp(i+1,2)=newnode(2);
        
        % Adds nodes for rest of wall to list
        for tv=i:right
            temp(tv,1)=node(tv,1);
            temp(tv,2)=node(tv,2);
        end
        
        % Connects wall elements together
        for cq=1:right-1
            elem(cq,1)=cq;
            elem(cq,2)=cq+1;
        end
        
        % Adds Bubble Nodes to temp list
        for vg=right+1:nnodes
            temp(vg,1)=node(vg,1);
            temp(vg,2)=node(vg,2);
        end
        
        % Turns templist into main node list
        node=temp;
        nnodes=size(node,1);
        
        % Connect Droplet Elements
        for k=right:(nnodes-2)
            elem(k,1)=k+1;
            elem(k,2)=k+2;
        end
        % Connect last droplet node to first droplet node
        elem(nnodes-1,1) = nnodes;
        elem(nnodes-1,2) = right+1;
        nelems=size(elem,1);
        right=right+1; % Recorrect right endpoint position
    % Standard case where bubble hasn't formed yet so right=nnodes
    else
        temp=zeros(nnodes+1,2);
    
        temp(1:node1,:)=node(1:node1,:);    
        temp(node1+1,1)=(x1+x2)/2;
        temp(node1+1,2)=(y1+y2)/2;    
        temp(node2+1:nnodes+1,:)=node(node2:nnodes,:);
        node=temp;

        elem(nelems,2)=nelems+1;
        elem(nelems+1,1)=nelems+1;
        elem(nelems+1,2)=nelems+2; 

        nelems = size(elem,1);
        nnodes = size(node,1);
    end

    
end
